import os, csv
import time
import arcpy
from arcpy import env
from arcpy.sa import *
arcpy.CheckOutExtension("Spatial")
arcpy.env.overwriteOutput=True

directory="C:/Users/00jdunca/OfflineDocuments/SPEI"
speilayer="spei_layer"
spei_vt="spei_vermont.tif"
bandstats="bandStats.txt"
#the shape used to extract the final snow raster for each day
vermontClip="I:/Geodata/VMCMasterGeodata.gdb/BaseInformation_VSP/VermontBoundary_VCGITownDissolve"
longtermmean="longtermmean.tif"
netcdf="spei06.nc" #6-month SPEI netcdf file
bandTextList=[] #holds the list of bands to use in the long-term mean calculation
yearMeans={} #Holds the yearly means

#change to that directory and set arc workspace
os.chdir(directory)
env.workspace=directory

#Open a file that gives the band number and the corresponding years and months, picking out the april bands
#Note that we exclude 1950 because it appears to be empty and cannot calculate statistics on it.
with open('spei_yearmonthdict.txt','r') as inf:
    dict_from_file = eval(inf.read())

#the envelope containing vermont, with some wiggle room. This is used to do an initial extraction of the continental dataset so we're only working with a subset to start.
roughRectangle="-74 42 -71 46"


arcpy.MakeNetCDFRasterLayer_md(in_netCDF_file=netcdf, variable="spei", x_dimension="lon", y_dimension="lat", out_raster_layer=speilayer, band_dimension="time", dimension_values="", value_selection_method="BY_VALUE")

#clip it to the extent of interest
print "clipping dataset"
arcpy.gp.ExtractByMask_sa(speilayer, vermontClip, spei_vt)
			

print "computing means for each band"
#Compute means for each band
#Loop through and get the list of band layers we need for statistics
for band in dict_from_file:
	if band['month']==4:
		print "Processing "+str(band['year'])+str(band['month'])
		#add the band number to the list of bands to use
		#We are only using the climate baseline of 1961 to 1990, so limit to that 
		if (band['year']>=1961 and band['year']<=1990):
			bandTextList.append(spei_vt+"/Band_"+str(band['bandNum']))
		#compute the mean for this year

		yearresult = arcpy.GetRasterProperties_management(spei_vt+"/Band_"+str(band['bandNum']), 'MEAN')
		yearMeans[band['year']]=yearresult.getOutput(0)
		
		
bandStatsText=";".join(bandTextList)

print "Computing long-term mean"
#Compute long-term means for each pixel
arcpy.gp.CellStatistics_sa(bandStatsText, longtermmean,"MEAN", "DATA")
print "getting the mean"
#Get the overall mean of the raster
result = arcpy.GetRasterProperties_management(longtermmean, 'MEAN')
#save that number as the long term mean for SPEI
longTermMean = result.getOutput(0)

#Write the output to a file
print "writing to file"
with open('spei.csv', 'wb') as csvfile:
	speicsv = csv.writer(csvfile, delimiter=',',quotechar='|', quoting=csv.QUOTE_MINIMAL)
	speicsv.writerow(['Year','YrMean','LTMean'])
	for y,t in  yearMeans.items():        
		speicsv.writerow([y, t,longTermMean])